% *************************************************************
% GABSOBHIN - Genetic Algorithms Based Synthesis & Optimization
% of Batch Heat Integration Networks - Copyright LENI-EPFL 2000
% Development Pierre KRUMMENACHER & Blaise RENAUD for LENI-EPFL 
% *************************************************************
% I_Heat_Recovery.m /  01.03.01 - .........
% *************************************************************
% Calculate equivalent (actual) heat recovery HR 
% *************************************************************
% INPUT:
%		:	
%
% OUTPUT:
%		:	
% *************************************************************
% to do:
%
% INTERNAL VARIABLES
%		:	
% *************************************************************

if PSOS.help_print == 1
   help I_Heat_Recovery
end

if PSOS.step_print == 1
   disp('*** I_Heat_Recovery ***');
end



%	Globally, without any heat integration, and provided that the HSUs are balanced over a batch cycle (i.e.
%	there is no energy state difference between the beginning and the end of a batch cycle), the (minimum)
%	required utilities to satisfy the process is given by the required heat to heat cold compulsory process
%	streams (i.e. CPS.CCC(CPS.CCC_size).H), and the required cooling to cool down hot compulsory process
%	streams (i.e. - HPS.CCC(HPS.CCC_size).H)

%	It is clear that the heat recovery cannot be defined as the heat recovered from hot stream to the HSUs,
%	or to that supplied by HSUs to cold streams, since these are generally not in balance and requiring mass
%	and heat balancing.

%	The actual heat recovery is therefore defined as the amount of heat which has not to be supplied by
%	the hot utility (hot utility savings), or the amount of cooling which has not to be supplied by the cold
%	utility (cold utility savings). This holds as long as no utility is applied on soft_T segments (which is
%	the case here).

%	To be general, one has to make sure the compulsory composite actually exists (note it is clear that at
%	least one of both CCC_size has to be >0, otherwise the problem does not make sense)

RD.HU_tot = sum(HUS.str(1).Q_BU) + sum(HUS.str(1).Q_PS);
RD.CU_tot = sum(CUS.str(1).Q_BU) + sum(CUS.str(1).Q_PS);

if HPS.CCC_size > 0 %	and for whatever CPS.CCC_size
   RD.HR = (- HPS.CCC(HPS.CCC_size).H) - RD.CU_tot;
else	%	i.e. HPS.CCC_size = 0
   if CPS.CCC_size > 0
      RD.HR = CPS.CCC(CPS.CCC_size).H - RD.HU_tot;
   end
end

%	dispnum(RD.HR,'RD.HR');
%	RD

% *************************************************************
