% *************************************************************
% GABSOBHIN - Genetic Algorithms Based Synthesis & Optimization
% of Batch Heat Integration Networks - Copyright LENI-EPFL 2000
% Development Pierre KRUMMENACHER & Blaise RENAUD for LENI-EPFL 
% *************************************************************
% I_Report_Evolution.m /  02.03.01 - .........
% *************************************************************
% Report current results (print & plot) during optimization
% *************************************************************
% INPUT:
%	OF_best	:	best value of current objective function. Basically
%					TBCs (hence TBCs_min), but also -HR (hence -HR_max)
%	OF_mean	:	average value, over population, of OF
%	Side_Val_of_OF_best	:	an important value (function) associated
%								:	such as HR (an important criteria for
%								:	evaluating an IHRS). Currently a value
%								:	read as the 6th element of the returned
%								:	yret1 vector to optimize at the end of ROFm
%
% OUTPUT:
%		:	
% *************************************************************
% to do:
%
% INTERNAL VARIABLES
%		:	
% *************************************************************
%	Note: at present, the Matlab module name "RAve" is hard coded in optimize.dll, i.e. optimize requires
%	the execution of the module called RAve (mex files is a Matlab module, not a Matlab function) and runs
%	the function found in this module (whatever the name of the function).

%	RAve called once for each generation (?), but plots are only refreshed every PSOS.Dgen_plot generation
%	using the drawnow function.

%	Note that RAve does not only plot data supplied by optimize.dll as input parameters, but is also used
%	to plot data available to RAve as the global structure RD (such as RD.TBCs_vs_HR, of RD.Best_IHRS, 
%	RD.Best_IHRS). Therefore RAve is not yet a very clean module with a clearly defined interface.

%	Note on the graphs : the mean value of TBCs plotted in the upper graph as a black cross (03.03.01: has been suppressed) is the mean value
%	of all generated individuals "falling" in that heat recovery range, while the mean value on lower graphs 
%	is the mean value over the population of that generation !!!

function [yret1] = Report_Evolution (OF_best, OF_mean, Side_Val_of_OF_best, no_gen)

%	Meaning and processing of input parameters to Report_Evolution OF_best, OF_mean, Side_Val_of_OF_best is
%	explained in I_Exec_DLL. The 4th input parameter is a GAmain internal parameter representing the number
%	(index) of the current generation.

global PSOS
global RD		%	to access RD.TBCs_vs_HR data
global DU

%	PSOS.NGen			total number of generations
%	PSOS.Dgen_plot		defining how many generations between 2 plot refresh (calculated in I_Report_Evol_Settings)
%	RD.ngen				equivalent number of generations calculated in I_Results_Update (RD.ngen=0: initial generation)

if PSOS.help_print == 1
   help RAve
end

if PSOS.step_print == 1
   disp('*** RAve ***');
end


%	make sure no_gen is the same as ngen calculated in ROFm:
if no_gen~=RD.ngen
   disp(['Error: no_gen supplied by optimize.dll (=',int2str(no_gen),') is not equal to ngen calculated in ROFm (='int2str(RD.ngen),') !']);
end

if PSOS.M_trace_print > 0
   dispnum ( RD.ngen, 'ngen', 'out of',num2str(PSOS.NGen) );
   
	dispnum ( OF_best, 'OF_best' );
	dispnum ( OF_mean, 'OF_mean' );
   dispnum ( Side_Val_of_OF_best, 'Side_Val_of_OF_best','=====> At end of RAve' );
   
	disp (' ');
else
	dispnum ( OF_best, 'OF_best', ['OF_mean = ', num2str(OF_mean), ' ** ngen = ', num2str(RD.ngen)], ['of ',num2str(PSOS.NGen),' <=*'] );
end


switch PSOS.OFType
case 1	%	minimization (in fine) of TBCs
   
	switch PSOS.OMode
	case 1	%	i.e. TBCs is minimized from the very beginning
         
   	%	update lower graph (total batch costs versus generations) at each generation, but the plot is actually
   	%	refreshed only every PSOS.Dgen_plot by the drawnow command.
      
   	subplot(2,1,2); hold on;
   	plot(RD.ngen, OF_best, 'b.',RD.ngen, OF_mean, 'k+');


   	%	update RD.TBCs_vs_HR plot of generated IHRS (upper graph). This is only done every PSOS.Dgen_plot
   	%	generation, since, unlike other plots, this plot is fully updated (points are not progressively added)
   	%	i.e. it is useless spending time updating at each generation for only keeping that of the last of 
   	%	the PSOS.Dgen_plot generation.
   
		if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen 	%	i.e. if RD.ngen is a multiple value of PSOS.Dgen_plot
      
      	subplot(2,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      	%	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      	plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
      	hold on;	%	hold on because more points are going to be added
      	%	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      	axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   		grid on;
   		xlabel(DU.HR_axis_label);
   		ylabel(DU.TBCs_axis_label);

      
 	   	for HR_index = 1:RD.N_HR+1
    			plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
      	end
         
      	drawnow;	%	drawnow refreshes the plot (between two drawnow commands, points to plot are stored but wait a drawnow to be displayed
      
   	end
   

   
   	%	to make sure that plots are refreshed for the last generation even in case PSOS.NGen is not a multiple
   	%	value of PSOS.Dgen_plot, the if statement below has to be placed outside the statement (namely
   	%	if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen):
   
   	if RD.ngen == PSOS.NGen	%	the last generation has been reached
      
      	subplot(2,1,2); hold on;
      
      	%	highlight the best OF identified
      	plot(RD.ngen, OF_best, 'r*');
      
      
      	subplot(2,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      	%	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      	plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
      	hold on;	%	hold on because more points are going to be added
      
      	%	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      	axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   		grid on;
   		xlabel(DU.HR_axis_label);
   		ylabel(DU.TBCs_axis_label);
      
 	   	for HR_index = 1:RD.N_HR+1
    			plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
      	end
      
         
      	%	plot TBCs & HR of "The Best Individual"
      	plot(Side_Val_of_OF_best, OF_best,'r*');
      
      	text (2/100 *(PSOS.HR_ul - PSOS.HR_ll),3/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['TBCs min = ', num2str(OF_best),DU.cost_B]);
      	text (48/100 *(PSOS.HR_ul - PSOS.HR_ll),1/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['HR of TBCs min = ', num2str(Side_Val_of_OF_best),DU.heat_kWh_B]);
      
            
      	drawnow	%	for a last refresh of plots at the end of the last generation
         
   	end	%	if RD.ngen == PSOS.NGen
   
	case 2	%	during a first phase, HR is maximized (RD.alpha=0), followed by a blend of HR and TBCs
   	%	(0<RD.alpha<1), and finally minimization of TBCs (RD.alpha=1)
      
      if RD.alpha == 0	%	i.e. RD.ngen <= ceil(PSOS.PMOF_HR * PSOS.NGen), i.e. maximizing HR
               
      	%	update middle graph (heat recovery versus generations) at each generation, but the plot is actually
      	%	refreshed only every PSOS.Dgen_plot by the drawnow command.
      
      	subplot(3,1,2); hold on;
      	plot(RD.ngen, -(OF_best - PSOS.MF_HR_max_TBCs_max * (RD.HR_max + RD.TBCs_max_approx)), 'b.',RD.ngen, -(OF_mean - PSOS.MF_HR_max_TBCs_max * (RD.HR_max + RD.TBCs_max_approx)), 'k+');
         %	-(OF_best-PSOS.MF_HR_max_TBCs_max*(RD.HR_max + RD.TBCs_max_approx) to display actual HR
         
      	%	plot on the TBCs versus Generation graph the TBCs of best HR (OF during this phase)
      
      	subplot(3,1,3); hold on;
         plot(RD.ngen, Side_Val_of_OF_best, 'b.');
         
      
      	if RD.ngen == ceil(PSOS.PMOF_HR * PSOS.NGen)	%	register the highest HR found to display the highest limit found
            RD.highest_HR = -(OF_best - PSOS.MF_HR_max_TBCs_max * (RD.HR_max + RD.TBCs_max_approx));
            %	note: RD.highest_HR initialized to 0 in I_Init, so that if PSOS.PMOF_HR=0 and we don't go
            %	through the first phase, we can check on the value of RD.highest_HR
      	end
      
      else
         if (RD.alpha > 0) & (RD.alpha < 1)	%	(RD.alpha > 0) is not necessary, but makes condition clear
            %	second step: maximization of a blend of HR & TBCs
            
            %	update middle graph (heat recovery versus generations) at each generation, but the plot is actually
            %	refreshed only every PSOS.Dgen_plot by the drawnow command.
            %	note that since this phase does not maximize HR, but a blend of HR & TBCs, the back-calculated
            %	value of HR from OF_best (best blend for this RD.alpha) is clearly not the highest HR, but the
            %	one which maximize the blended HR & TBCs OF function
      
      		subplot(3,1,2); hold on;
      		plot(RD.ngen, -(1 /(1 - RD.alpha)) * (OF_best - RD.alpha * Side_Val_of_OF_best - (1 - RD.alpha) * PSOS.MF_HR_max_TBCs_max * (RD.HR_max + RD.TBCs_max_approx)), 'r.');	% to display actual HR
            
            if RD.highest_HR ~=0	%	i.e. we went through the first phase
               plot(RD.ngen, RD.highest_HR, 'b.');
               
            end
            
      		%	plot on the TBCs versus Generation graph the TBCs of best blended HR & TBCs (OF during this phase)
            %	according to the note above, the TBCs returned as Side_Val_of_OF_best is not the cost of the
            %	highest HR !!!
            
            subplot(3,1,3); hold on;
         	plot(RD.ngen, Side_Val_of_OF_best, 'b.');
            
         else	%	i.e. RD.alpha = 1	=> no OF back-calculation to obtain the actual TBCs
            %	update lower graph (total batch costs versus generations) at each generation, but the plot is actually
      		%	refreshed only every PSOS.Dgen_plot by the drawnow command.
      
      		subplot(3,1,3); hold on;
      		plot(RD.ngen, OF_best, 'b.',RD.ngen, OF_mean, 'k+');
      
      
      		%	plot the evolution of solutions in the HR versus Generation graph:
      
      		subplot(3,1,2); hold on;
            plot(RD.ngen, Side_Val_of_OF_best, 'r.');
            
            if RD.highest_HR ~=0	%	i.e. we went through the first phase
               plot(RD.ngen, RD.highest_HR, 'b.');
               
            end
         end
      
   	end	%	check on the value of RD.alpha


   	%	update TBCs_vs_HR plot of generated IHRS (upper graph). This is only done every PSOS.Dgen_plot
   	%	generation, since, unlike other plots, this plot is fully updated (points are not progressively added)
   	%	i.e. it is useless spending time updating at each generation for only keeping that of the last of 
   	%	the PSOS.Dgen_plot generation.
   
		if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen 	%	i.e. if RD.ngen is a multiple value of PSOS.Dgen_plot
      
      	subplot(3,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      	%	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      	plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
      	hold on;	%	hold on because more points are going to be added
      
      	%	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      	axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   		grid on;
   		xlabel(DU.HR_axis_label);
   		ylabel(DU.TBCs_axis_label);
      
      	for HR_index = 1:RD.N_HR+1
    			plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
      	end
         
      	drawnow;	%	drawnow refreshes the plot (between two drawnow commands, points to plot are stored but wait a drawnow to be displayed
      
   	end
   

   
   	%	to make sure that plots are refreshed for the last generation even in case PSOS.NGen is not a multiple
   	%	value of PSOS.Dgen_plot, the if statement below has to be placed outside the statement (namely
   	%	if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen):
   
   	if RD.ngen == PSOS.NGen	%	the last generation has been reached
      
      	subplot(3,1,3); hold on;
      
      	%	highlight the best OF identified
      	plot(RD.ngen, OF_best, 'r*');

      
      	subplot(3,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      	%	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      	plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
      	hold on;	%	hold on because more points are going to be added
      
      	%	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      	axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   		grid on;
   		xlabel(DU.HR_axis_label);
   		ylabel(DU.TBCs_axis_label);
      
 	   	for HR_index = 1:RD.N_HR+1
    			plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
      	end
      
      
      	%	plot TBCs & HR of "The Best Individual"
      	plot(Side_Val_of_OF_best, OF_best,'r*');
      
      	text (2/100 *(PSOS.HR_ul - PSOS.HR_ll),3/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['TBCs min = ', num2str(OF_best),DU.cost_B]);
      	text (48/100 *(PSOS.HR_ul - PSOS.HR_ll),1/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['HR of TBCs min = ', num2str(Side_Val_of_OF_best),DU.heat_kWh_B]);
            
      	drawnow	%	for a last refresh of plots at the end of the last generation
         
   	end	%	if RD.ngen == PSOS.NGen
  

	otherwise
   	%	not a case to be found
   end	%	switch PSOS.OMode
   
case 2	%	maximization of HR
   
   %	update lower graph (heat recovery versus generations) at each generation, but the plot is actually
   %	refreshed only every PSOS.Dgen_plot by the drawnow command.
      
  	subplot(2,1,2); hold on;
   plot(RD.ngen, -OF_best, 'b.',RD.ngen, -OF_mean, 'k+');


   %	update RD.TBCs_vs_HR plot of generated IHRS (upper graph). This is only done every PSOS.Dgen_plot
   %	generation, since, unlike other plots, this plot is fully updated (points are not progressively added)
   %	i.e. it is useless spending time updating at each generation for only keeping that of the last of 
   %	the PSOS.Dgen_plot generation.
   
	if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen 	%	i.e. if RD.ngen is a multiple value of PSOS.Dgen_plot
      
      subplot(2,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      	%	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
     	hold on;	%	hold on because more points are going to be added
      	%	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   	grid on;
   	xlabel(DU.HR_axis_label);
   	ylabel(DU.TBCs_axis_label);

      
 	   for HR_index = 1:RD.N_HR+1
    		plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
      end
         
      drawnow;	%	drawnow refreshes the plot (between two drawnow commands, points to plot are stored but wait a drawnow to be displayed
      
  	end
   

   
   %	to make sure that plots are refreshed for the last generation even in case PSOS.NGen is not a multiple
   %	value of PSOS.Dgen_plot, the if statement below has to be placed outside the statement (namely
   %	if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen):
   
   if RD.ngen == PSOS.NGen	%	the last generation has been reached
      
      subplot(2,1,2); hold on;
      
      %	highlight the best OF identified
      plot(RD.ngen, -OF_best, 'r*');
      
      
     	subplot(2,1,1); hold off;	%	hold off because the previous points on the graph need to be removed
      %	when plotting the two points below (even if these are anyway plotted again by the for ... statement)
      plot(PSOS.HR_ll, RD.TBCs_vs_HR(1).TBCs_min, 'b.');
      hold on;	%	hold on because more points are going to be added
      
      %	reformat the graph, otherwise previous hold off suppress scale, grid, labels, etc.
      axis([PSOS.HR_ll PSOS.HR_ul PSOS.TBCs_ll PSOS.TBCs_ul]);
   	grid on;
   	xlabel(DU.HR_axis_label);
      ylabel(DU.TBCs_axis_label);
      
 	   for HR_index = 1:RD.N_HR+1
    		plot(PSOS.HR_ll + (HR_index-1)*PSOS.delta_HR, RD.TBCs_vs_HR(HR_index).TBCs_min, 'b.');
     	end
        
               
     	%	plot TBCs & HR of "The Best Individual"
      plot(-OF_best, Side_Val_of_OF_best, 'r*');
      
      text (2/100 *(PSOS.HR_ul - PSOS.HR_ll),1/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['HR max = ', num2str(-OF_best),DU.heat_kWh_B]);
      text (48/100 *(PSOS.HR_ul - PSOS.HR_ll),1/10*(PSOS.TBCs_ul - PSOS.TBCs_ll),['TBCs of HR max = ', num2str(Side_Val_of_OF_best),DU.cost_B]);
            
      drawnow	%	for a last refresh of plots at the end of the last generation
         
   end	%	if RD.ngen == PSOS.NGen
   
   
otherwise
   %	not a case to be found
   
end	%	switch PSOS.OFType



yret1 = 989;


if (floor(RD.ngen/PSOS.Dgen_plot)*PSOS.Dgen_plot) == RD.ngen 	%	i.e. if RD.ngen is a multiple value of PSOS.Dgen_plot
	if PSOS.M_trace_print > 0
		dispnum ( floor(RD.ngen/PSOS.Dgen_plot),'floor(RD.ngen/PSOS.Dgen_plot)');
	end
end
